import pygame
from pygame import *

class Screen:
  def __init__(self, (x, y), fullscreen = False):
    self.x, self.y = x, y
    self.fullscreen = fullscreen
    self.setmode()
    self.wallpaper = None
  def setmode(self):
    if self.fullscreen:
      self.surf = display.set_mode(self.nearestfmode(), FULLSCREEN | DOUBLEBUF)
    else:
      self.surf = display.set_mode((self.x, self.y), DOUBLEBUF)
  # Nearest fullscreen mode to current res
  def nearestfmode(self):
    print display.list_modes()
    modes = display.list_modes()
    mup = [(x, y) for (x, y) in modes if x >= self.x and y >= self.y]
    return mup[-1] if mup else modes[0]
  def setwallpaper(self, wallpaper):
    self.wallpaper = wallpaper
    self.wx, self.wy = wallpaper.get_size()
  # Try to center the given coordinates as best you can
  def orient(self, (xc, yc) = (None, None)):
    minmax = lambda lo, x, hi: min(max(lo, x), hi)
    if self.wx > self.x and xc is not None:
      self.x0 = minmax(self.x - self.wx, self.x/2 - xc, 0)
    else:
      self.x0 = (self.x - self.wx) / 2
    if self.wy > self.y and yc is not None:
      self.y0 = minmax(self.y - self.wy, self.y/2 - yc, 0)
    else:
      self.y0 = (self.y - self.wy) / 2
  def pos(self, (x, y)):
    return int(x + self.x0), int(y + self.y0)
  def tpos(self, (x, y)):
    return x - self.x0, y - self.y0
  def clear(self):
    if self.wx < self.x or self.wy < self.y:
      self.surf.fill((0,0,0))
    self.surf.blit(self.wallpaper, self.pos((0, 0)))
  def onscreen(self, (x, y), m = 0):
    return m <= x <= self.wx - m and m <= y <= self.wy - m
  def whichdir(self, (x, y), m = 0):
    if y < m: return 0
    if x > self.wx - m: return 1
    if y > self.wy - m: return 2
    if x < m: return 3
    return None


