import pygame, data, trajectory, wallseg
from pygame import *
from trajectory import *
from wallseg import *


class projectile:
  def __init__(self, traj, ws = [], r = 1):
    self.traj = traj
    self.walls = ws
    self.age = 0
    self.ticker = 0  # time since last bounce
    self.setbounce()
    self.checkbounce()
    self.setpos()
    self.r = r
  def setpos(self):
    self.pos = self.traj.at(self.ticker)
  def setbounce(self):
    self.tbounce, self.ntraj, self.wbounce = None, None, None
    for w in self.walls:
      c = w.catchtraj(self.traj)
      if c and (self.tbounce is None or c[0] < self.tbounce):
        self.tbounce, self.ntraj = c
        self.wbounce = w
  def checkbounce(self):
    while self.tbounce is not None and self.ticker > self.tbounce:
      self.ticker -= self.tbounce
      self.traj = self.ntraj
      self.setbounce()
  def update(self, dt):
    self.ticker += dt
    self.age += dt
    self.checkbounce()
    self.setpos()
  def draw(self, scr):
    draw.circle(scr.surf, (255, 255, 255), screenpos(self.pos), self.r)

class tracer (projectile):
  def __init__(self, traj, ws, ltime):
    self.traj = traj
    self.walls = ws
    self.age = 0
    self.r = 0
    self.ticker = ltime
    self.getps()
  def getps(self):
    self.ps = [self.traj.at(0.1)]
    self.setbounce()
    while self.tbounce is not None and self.ticker > self.tbounce:
      self.ticker -= self.tbounce
      self.traj = self.ntraj
      self.ps.append(self.traj.at(0))
      self.setbounce()
    self.ps.append(self.traj.at(self.ticker))
  def willhit(self, target, r0 = 12):
    r = target.r + r0
    for j in range(1,len(self.ps)-1):  # only after the first bounce
      p0, p1 = self.ps[j], self.ps[j+1]
      d = wallseg(p0, p1).dtosprite(target)
      if d is not None and abs(d) < r: return True
    return False      
  def draw(self, scr):
    for j in range(len(self.ps) - 1):
      draw.line(scr.surf, (255, 255, 255), scr.pos(self.ps[j]), scr.pos(self.ps[j+1]), 1)

class spell (projectile):
  def __init__(self, traj, ws = [], r = 12):
    projectile.__init__(self, traj, ws, r)
    if not self.frame: self.loadimages()
    self.jframe = 0
  @classmethod
  def loadimages(cls):
    frame0 = data.loadimg("spells", cls.name + ".png")
    cls.frame = [transform.rotate(frame0, a) for a in (0, 90, 180, 270)]
    cls.icon = data.loadimg("feathers", cls.name + ".png")
    font.init()
    f = font.SysFont("freeserif", 20, True, True)
    cls.caption = f.render(cls.name, True, (0, 0, 0), (255, 255, 255))
  def update(self, dt):
    projectile.update(self, dt)
    self.jframe += dt * 16
  def draw(self, scr):
    frame = self.frame[int(self.jframe) % len(self.frame)]
    pos = self.pos[0] - 16, self.pos[1] - 16
    scr.surf.blit(frame, scr.pos(pos))
  def afflict(self, target):
    pass

class fspell(spell):
  frame = None
  name = "falcon"
  icon = None
  def afflict(self, target):
    target.harm(1, self)

class dspell(spell):
  frame = None
  name = "dove"
  icon = None
  def afflict(self, target):
    target.naptime()

class hspell(spell):
  frame = None
  name = "hummingbird"
  icon = None
  def afflict(self, target):
    target.humming = 9

class ospell(spell):
  frame = None
  name = "owl"
  icon = None
  def afflict(self, target):
    target.tracer = 5

class rspell(spell):
  frame = None
  name = "raven"
  icon = None
  def afflict(self, target):
    target.nthrash = 20

class pspell(spell):
  frame = None
  name = "peacock"
  icon = None
  def afflict(self, target):
    target.zapcount = 2

