# Mah sprite class
import pygame, random, data, math, spell, rimage, sound
from pygame import *
from random import *
from math import *
from spell import *

class thing:
  hframe = None
  def __init__(self, pos, r, v, hp):
    if not self.hframe: self.makehframes()
    self.px, self.py = self.pos = pos
    self.pursuit = None
    self.r, self.v = r, v
    self.hp = hp
    self.tharm = 0  # How long to show harming animation
    self.alpha = 0
    self.alive = True
    self.reeling = 0
  @staticmethod
  def makehframes():
    slash = data.loadimg("slash.png")
    thing.hframe = []
    nframes = 12
    for j in range(nframes):
      f = Surface((48, 48)).convert_alpha()
      f.fill((0,0,0,0))
      x = 48 - j * 96 / nframes
      f.blit(slash, (x, 0), Rect(x, 0, 48, 48))
      thing.hframe.append(f)
    nframes = 11
    thing.zframe = [data.loadimg("zed-%d.png" % j) for j in range(nframes)]
  def setpos(self, (x, y)):
    self.px, self.py = self.pos = x, y
  def shift(self, (dx, dy)):
    self.px += dx
    self.py += dy
    self.pos = self.px, self.py
  def setpursuit(self, p):
    self.pursuit = p
    self.alpha = atan2(self.px - p[0], self.py - p[1]) / (2 * pi)
  def harm(self, amt = 1, other = None):
    self.hp -= amt
    self.tharm = 0.3
    self.tsleep = 0
    if other: self.reel(other.pos)
    sound.playsound("slash")
  def reel(self, (x, y)):
    self.reeling = 0.2
    dx, dy = self.px - x, self.py - y
    if (dx, dy) == (0, 0): dy = 1
    d = sqrt(dx ** 2 + dy ** 2)
    self.reeldir = dx / d, dy / d
  def update(self, dt):
    if self.pursuit:
      d = self.v * dt
      dx, dy = self.pursuit[0] - self.px, self.pursuit[1] - self.py
      dist = sqrt(dx ** 2 + dy ** 2)
      if dist < d:
        self.px, self.py = self.pos = self.pursuit
        self.pursuit = None
      else:
        self.shift((dx * d / dist, dy * d / dist))
    if self.reeling:
      self.shift((dt * self.reeldir[0] * 400, dt * self.reeldir[1] * 400))
    if self.nthrash and not self.reeling:
      self.reeling = 0.1
      self.nthrash -= 1
      self.alpha = uniform(0, 1)
      alpha = uniform(0, 2 * pi)
      self.reeldir = sin(alpha), cos(alpha)
    if self.tharm:
      self.tharm -= dt
      if self.tharm <= 0:
        self.tharm = 0
        if self.hp <= 0:
          self.alive = False
  def randompursue(self, r):
    p = self.px + uniform(-200, 200), self.py + uniform(-200, 200)
    if self.cansee(p, r):
      self.setpursuit(p)
  def trajtoward(self, (x, y), v):
    dx, dy = x - self.px, y - self.py
    if dx == 0 and dy == 0: dy = 1
    d = sqrt(dx ** 2 + dy ** 2)
    return trajectory(self.pos, (dx * v / d, dy * v / d))
  def cansee(self, (x, y), room):
    traj = trajectory(self.pos, (x - self.pos[0], y - self.pos[1]))
    for w in room.walls:
      c = w.catchtraj(traj)
      if c and c[0] < 1.: return False
    return True
  def drawharm(self, scr):
    if not self.tharm: return False
    j = int((1 - self.tharm / 0.3) * len(self.hframe))
    if 0 <= j < len(self.hframe):
      p = self.pos[0] - 24, self.pos[1] - 24
      scr.surf.blit(self.hframe[j], scr.pos(p))
  def drawsleep(self, scr):
    if not self.tsleep: return False
    j = int((1 + sin(self.tsleep * 2 * pi)) / 2 * len(self.zframe))
    j = min(max(j, 0), len(self.zframe)-1)
    r = self.zframe[j].get_rect()
    r.bottomleft = scr.pos(self.pos)
    scr.surf.blit(self.zframe[j], r)
  def pic(self):
    if self.tsleep:
      j = 0
    elif self.attacking:
      j = self.aseq[int((self.atime - self.attacking) * self.afcount) % len(self.aseq)]
    elif self.patrolling:  # Walking
      j = self.wseq[int(self.jwalk * self.wfcount) % len(self.wseq)]
    elif self.pursuit:  # running
      j = self.rseq[int(self.jwalk * self.rfcount) % len(self.rseq)]
    else:
      j = 0
    return self.images[j].getframe(self.alpha)
  def cancast(self):
    return False

class wizard (thing):
  loaded = False
  dname = "you2"
  castcount0 = 1.
  npic = 5
  wseq = (0, 1, 4, 1, 0, 2, 3, 2)
  wfcount = 12
  aseq = (0,)
  afcount = 1
  atime = 1.
  hp0 = 3
  def __init__(self, pos):
    thing.__init__(self, pos, 20, 200, self.hp0)
    self.castcount = 0
    self.humming = 0
    self.tsleep = 0
    self.jwalk = 0
    self.attacking = 0
    self.tracer = 0
    self.reeling = 0
    self.thrashing = 0
    self.nthrash = 0
    if not self.loaded: self.loadimages()
  @classmethod
  def loadimages(cls):
    cls.heart0 = data.loadimg("heart0.png")
    cls.heart1 = data.loadimg("heart1.png")
    if cls.dname:
      cls.images = [rimage.Rimage(cls.dname, "p" + str(j) + "-") for j in range(cls.npic)]
      cls.rpic = cls.images[0].frame[0].get_width() / 2
    cls.loaded = True
  def update(self, dt):
    if self.humming: dt *= 3
    thing.update(self, dt)
    if not self.cancast():
      self.castcount -= dt
      if self.castcount < 0: self.castcount = 0
    if self.tsleep:
      self.tsleep = max(self.tsleep - dt, 0)
    if self.humming:
      self.humming = max(self.humming - dt, 0)
    if self.attacking:
      self.attacking = max(self.attacking - dt, 0)
    if self.tracer:
      self.tracer = max(self.tracer - dt, 0)
    if self.reeling:
      self.reeling = max(self.reeling - dt, 0)
    self.jwalk += dt
  def cancast(self):
    return self.castcount == 0 and not self.attacking and not self.tsleep
  def casttoward(self, p, room, stype):
    self.attacking = self.atime
    self.alpha = atan2(self.px - p[0], self.py - p[1]) / (2 * pi)
    self.castcount = self.castcount0
    traj = self.trajtoward(p, 300.)
    s = stype(traj, room.walls)
    s.update(0.2)
    return s
  def draw(self, scr):
    if self.tsleep:
      j = 0
    else:
      j = self.wseq[int(self.jwalk * self.wfcount) % len(self.wseq)]
    pic = self.images[j].getframe(self.alpha)
    scr.surf.blit(pic, scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)
  def naptime(self):
    self.tsleep = 3
    self.pursuit = False

class larry(wizard):
  loaded = False
  dname = "larry"
  castcount0 = 1.
  npic = 4
  wseq = (0, 1, 0, 2)
  wfcount = 10
  rseq = (0, 1, 0, 2)
  rfcount = 10
  aseq = (3, 0)
  afcount = 1
  atime = 1.
  hp0 = 3
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.cansee(you.pos, room) and not self.tsleep and not self.attacking and self.cancast():
      sound.playsound("cast")
      return self.casttoward(you.pos, room, fspell)
    if not self.pursuit and not self.attacking and not self.tsleep and uniform(0, 0.4) < dt:
      self.randompursue(room)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)

class morgan(wizard):
  loaded = False
  dname = "morgan"
  castcount0 = 2.
  npic = 4
  wseq = (0, 1, 0, 2)
  wfcount = 2
  rseq = (0, 1, 0, 2)
  rfcount = 4
  aseq = (3, 0)
  afcount = 1
  atime = 1.
  hp0 = 5
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    seeyou = self.cansee(you.pos, room)
    if seeyou and self.cancast():
      sound.playsound("cast")
      return self.casttoward(you.pos, room, fspell)
    elif you.tsleep and not self.pursuit and not seeyou:
      atemp = self.alpha
      self.randompursue(room)
      if self.pursuit:
        ptemp = self.pos
        self.pos = self.px, self.py = self.pursuit
        if not self.cansee(you.pos, room):
          self.pursuit, self.alpha = None, atemp
        self.setpos(ptemp)
      return None
    elif not you.tsleep and not self.pursuit and self.cancast():
      pos = self.px + uniform(-100, 100), self.py + uniform(-100, 100)
      t = self.trajtoward(pos, 300)
      trac = tracer(t, room.walls, 10)
      if not trac.willhit(self) and trac.willhit(you):
        sound.playsound("cast")
        return self.casttoward(pos, room, dspell)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)


class adelpaite(wizard):
  loaded = False
  dname = "adelpaite"
  castcount0 = 1.
  npic = 1
  wseq = (0,)
  wfcount = 2
  rseq = (0,)
  rfcount = 4
  aseq = (0,)
  afcount = 1
  atime = 1.
  hp0 = 5
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    seeyou = self.cansee(you.pos, room)
    if seeyou and self.cancast():
      sound.playsound("cast")
      return self.casttoward(you.pos, room, fspell)
    elif not you.tsleep and not self.pursuit and self.cancast():
      pos = self.px + uniform(-100, 100), self.py + uniform(-100, 100)
      t = self.trajtoward(pos, 300)
      trac = tracer(t, room.walls, 10)
      if not trac.willhit(self) and trac.willhit(you):
        sound.playsound("cast")
        return self.casttoward(pos, room, fspell)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)

class azcoatl(wizard):
  loaded = False
  dname = "azcoatl"
  castcount0 = 0.6
  npic = 3
  wseq = (0, 1, 0, 2)
  wfcount = 8
  rseq = (0, 1, 0, 2)
  rfcount = 16
  aseq = (0, )
  afcount = 1
  atime = 0.6
  hp0 = 8
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
    self.v *= 3
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    if not self.pursuit:
      if self.cancast() and uniform(0, 1) < 0.9:
        sound.playsound("cast")
        return self.casttoward(you.pos, room, fspell)
      else:
        self.randompursue(room)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)


class sister(wizard):
  loaded = False
  dname = "sister"
  castcount0 = 4
  npic = 1
  wseq = (0,)
  wfcount = 1
  rseq = (0,)
  rfcount = 1
  aseq = (0, )
  afcount = 1
  atime = 1
  hp0 = 4
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    if not self.pursuit:
      if self.cancast() and self.cansee(you.pos, room):
        sound.playsound("cast")
        return self.casttoward(you.pos, room, rspell if uniform(0, 1) < 0.5 else fspell)
      else:
        self.randompursue(room)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)

class grigori(wizard):
  loaded = False
  dname = "grigori"
  castcount0 = 1
  npic = 3
  wseq = (0, 1, 0, 2)
  wfcount = 3
  rseq = (0, 1, 0, 2)
  rfcount = 6
  aseq = (0, )
  afcount = 1
  atime = 1
  hp0 = 12
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    if not self.pursuit:
      if self.cancast() and self.cansee(you.pos, room):
        sound.playsound("cast")
        return self.casttoward(you.pos, room, fspell)
      else:
        self.randompursue(room)
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    self.drawharm(scr)
    self.drawsleep(scr)

class priscilla(wizard):
  loaded = False
  dname = "priscilla"
  castcount0 = 1
  npic = 3
  wseq = (0, 1, 0, 2)
  wfcount = 3
  rseq = (0, 1, 0, 2)
  rfcount = 6
  aseq = (0, )
  afcount = 1
  atime = 1
  hp0 = 3
  def __init__(self, pos):
    wizard.__init__(self, pos)
    self.attacking = False
    self.patrolling = False
    self.zapcount = 0
  def update(self, dt, you, room):
    wizard.update(self, dt)
    if self.attacking or self.tsleep: return False
    if not self.pursuit:
      if self.cancast() and self.cansee(you.pos, room):
        sound.playsound("cast")
        return self.casttoward(you.pos, room, fspell)
      else:
        self.randompursue(room)
    if self.zapcount:
      self.pursuit = None
      self.zapcount -= dt
      if self.zapcount < 0: self.zapcount = 0
  def draw(self, scr):
    scr.surf.blit(self.pic(), scr.pos((self.pos[0] - self.rpic, self.pos[1] - self.rpic)))
    if self.zapcount:
      for j in (1, 2, 3):
        r = rect.Rect(0, 0, uniform(60, 200), uniform(60, 200))
        r.center = scr.pos(self.pos)
        draw.ellipse(scr.surf, (128, 128, 255), r, 2)
    self.drawharm(scr)
    self.drawsleep(scr)
  def harm(self, amt = 1, other = None):
    if self.zapcount:
      self.hp -= amt
      self.tharm = 0.3
      self.tsleep = 0
      if other: self.reel(other.pos)
      sound.playsound("slash")
      self.zapcount = 0
    else:
      sound.playsound("dink")


def collide(thing1, thing2):
  (x1, y1), (x2, y2) = thing1.pos, thing2.pos
  r = thing1.r + thing2.r
  dx, dy = x2 - x1, y2 - y1
  if abs(dx) > r or abs(dy) > r: return False
  return dx ** 2 + dy ** 2 < r ** 2


